﻿using UnityEngine;
using System.Collections;

public class GrabObject : MonoBehaviour {

	public GameObject hand;

	public GameObject gelTarget;
	public GameObject transduserTarget;
	public GameObject itemAreaTarget;
	public GameObject scanningAreaTarget;

	private GameObject grabbedObject = null;
	private Transform objectOrgParent = null;

	private bool itemInHand = false;
	private float coolDownTimer = 0.0f;

	public bool hasGelInHand = false;
	public bool hasTransducerInHand = false;
	public bool isWithinScanningArea = false;

	void onStart(){
		coolDownTimer = Time.time;
	}

	void OnTriggerExit(Collider collision) {

		isWithinScanningArea = false;
	}

	public void ResetGrabbing(){

		if (itemInHand) {
			//Remove item from hand and place it back onto the tray:
			grabbedObject.GetComponent<Rigidbody> ().isKinematic = false;
			grabbedObject.transform.parent = objectOrgParent;

			if (grabbedObject == gelTarget) {
				grabbedObject.transform.position = objectOrgParent.transform.position;
				grabbedObject.transform.rotation = objectOrgParent.transform.rotation;
				grabbedObject.transform.localPosition += new Vector3 (-0.11f, 0, -0.06f);
				grabbedObject.transform.Rotate (270, 270, 0);
			} else if (grabbedObject == transduserTarget) {
				grabbedObject.transform.position = objectOrgParent.transform.position;
				grabbedObject.transform.rotation = objectOrgParent.transform.rotation;
				grabbedObject.transform.localPosition += new Vector3 (-0.11f, 0, 0.06f);
				grabbedObject.transform.Rotate (0, 0, 0);
			}
			itemInHand = false;
			hasGelInHand = false;
			hasTransducerInHand = false;
			coolDownTimer = Time.time;
			return;
		}

		//This might possible be a bit of a bear.
		itemInHand = false;
		hasGelInHand = false;
		hasTransducerInHand = false;
		coolDownTimer = Time.time;
		return;
	}

	void OnTriggerEnter(Collider collision){
		//Debug.Log ("Grabbed something");

		//If there's no item in the hand
		if (!itemInHand) {
		
			//Check if the cooldown is done
			if ((Time.time - coolDownTimer) < 1.0f) {
				return;
			}

			//Check if the collision is the gel target or transducer
			if (collision.gameObject == gelTarget) {
				grabbedObject = collision.gameObject;
				//Check if the object is already being held.
				if (grabbedObject.GetComponent<Rigidbody> ().isKinematic) {
					return;
				}
				// Make it kinematic as we are holding it now
				grabbedObject.GetComponent<Rigidbody> ().isKinematic = true;
				// Store the original parent to restore it when letting loose
				objectOrgParent = grabbedObject.transform.parent;
				// And parent it to the hand

				//Position the object correctly in the hand
				grabbedObject.transform.parent = hand.transform;
				grabbedObject.transform.position = hand.transform.position;
				grabbedObject.transform.rotation = hand.transform.rotation;
				grabbedObject.transform.localPosition += new Vector3 (-0.02f, 0.1f, 0);
				grabbedObject.transform.Rotate (0, 180, 0);

				itemInHand = true;
				hasGelInHand = true;
				coolDownTimer = Time.time;
				return;

			} else if (collision.gameObject == transduserTarget) {
				grabbedObject = collision.gameObject;
				//Check if the object is already being held.
				if (grabbedObject.GetComponent<Rigidbody> ().isKinematic) {
					return;
				}
				// Make it kinematic as we are holding it now
				grabbedObject.GetComponent<Rigidbody> ().isKinematic = true;
				// Store the original parent to restore it when letting loose
				objectOrgParent = grabbedObject.transform.parent;
				// And parent it to the hand

				//Position the object correctly in the hand
				grabbedObject.transform.parent = hand.transform;
				grabbedObject.transform.position = hand.transform.position;
				grabbedObject.transform.rotation = hand.transform.rotation;
				grabbedObject.transform.localPosition += new Vector3 (-0.02f, 0.1f, 0);
				grabbedObject.transform.Rotate (-90, 0, 0);

				hasTransducerInHand = true;
				itemInHand = true;
				coolDownTimer = Time.time;
				return;
			}
		}

		//If there is an item in hand
		if (itemInHand) {

			//Check if the cooldown is done
			if ((Time.time - coolDownTimer) < 1.0f) {
				return;
			}

			if (collision.gameObject == itemAreaTarget){
				//Remove item from hand and place it back onto the tray:
				grabbedObject.GetComponent<Rigidbody> ().isKinematic = false;
				grabbedObject.transform.parent = objectOrgParent;

				if (grabbedObject == gelTarget) {
					grabbedObject.transform.position = objectOrgParent.transform.position;
					grabbedObject.transform.rotation = objectOrgParent.transform.rotation;
					grabbedObject.transform.localPosition += new Vector3 (-0.11f, 0, -0.06f);
					grabbedObject.transform.Rotate (270, 270, 0);
				} else if (grabbedObject == transduserTarget) {
					grabbedObject.transform.position = objectOrgParent.transform.position;
					grabbedObject.transform.rotation = objectOrgParent.transform.rotation;
					grabbedObject.transform.localPosition += new Vector3 (-0.11f, 0, 0.06f);
					grabbedObject.transform.Rotate (0, 0, 0);
				}
				itemInHand = false;
				hasGelInHand = false;
				hasTransducerInHand = false;
				coolDownTimer = Time.time;
				return;
			}
			if (collision.gameObject == scanningAreaTarget) {

				isWithinScanningArea = true;
				coolDownTimer = Time.time;
				return;
			}
		}

	}

}
