﻿using UnityEngine;
using System.Collections;
using UnityEngine.UI;

public class ScenarioScript : MonoBehaviour
{
	//Variables for the speech bubble.
	public Canvas speechCanvas; 
	private Text speechButtonText;
	private Text buttonOne;
	private Text buttonTwo;

	public Canvas dialogueCanvas;
	private ButtonHandler dialogueButtonScript;

	private Transform doctorsSpeechTransform;
	private Transform defaultUITransform;
	private Transform oppositeWallTransform;

	private GameObject camera;
	private GameObject UiCamera;
	private GameObject alexViewObject;

	public GameObject Alex;
	private GameObject AlexRenderer;
	private GameObject AlexMesh;
	private IKControlAlex alexIKScript;

	private GameObject AlexWithoutHeadMesh;

	public GameObject Doctor;
	private GameObject DoctorMesh;
	public GameObject DoctorWithHead;
	private GameObject DoctorWithHeadMesh;

	Texture alexGelledTex;
	Texture alexNormal;

	//Variables for hand
	public GameObject RightHand;
	private GrabObject RightHandScript;

	public GameObject LeftHand;
	private GrabObject LeftHandScript;

	private StoreMovementScript movScript;

	//Movie variables
	public GameObject moviePlane;
	private MovieTexture movie;

	int STATE = 1;
	const int STATE_START = 1;
	const int STATE_APPLY_GEL = 2;
	const int STATE_SCAN = 3;
	const int STATE_SCANNING = 4;
	const int STATE_QUESTION_FROM_ALEX = 5;
	const int STATE_ANSWER_FROM_DOCTOR = 6;
	const int STATE_LEARNING = 7;
	const int STATE_END = 8;

	private bool alexView = false;
	private bool setup = true;

	const int STATE_WAIT = 100;
	float timer = 0.0f;

	// Use this for initialization
	void Start ()
	{
		RightHandScript = RightHand.GetComponent<GrabObject>();
		LeftHandScript = LeftHand.GetComponent<GrabObject>();
		movScript = GetComponent<StoreMovementScript>();

		speechButtonText = speechCanvas.GetComponentsInChildren<Text> () [0];
		buttonOne = dialogueCanvas.GetComponentsInChildren<Text> () [0];
		buttonTwo = dialogueCanvas.GetComponentsInChildren<Text> () [1];

		dialogueButtonScript = dialogueCanvas.GetComponent<ButtonHandler> ();
		alexGelledTex = Resources.Load("AlexGelled") as Texture;
		alexNormal = Resources.Load("Alex") as Texture;

		DoctorMesh = GameObject.Find ("CarlWithBonesTest/CarlMidGeo");
		DoctorWithHeadMesh = GameObject.Find ("CarlWithBonesTestReservedModel/CarlMidGeoHead");

		AlexMesh = GameObject.Find("AlexWithBones/Alexis_Geo_Low");
		AlexWithoutHeadMesh = GameObject.Find("AlexWithBonesWithoutHead/Alexis_Geo_Low");
		alexIKScript = GameObject.Find ("AlexWithBones").GetComponent<IKControlAlex>();

		camera = GameObject.Find ("OVRCameraRig");
		UiCamera = GameObject.Find ("UICamera");
		alexViewObject = GameObject.Find ("AlexisView");

		doctorsSpeechTransform = GameObject.Find ("DoctorSpeechPlacement").GetComponent<Transform>();
		defaultUITransform = GameObject.Find ("PosDefaultUI").GetComponent<Transform>();
		oppositeWallTransform = GameObject.Find ("PosWallOppositeDoctor").GetComponent<Transform>();

		//Set up speech canvas
		speechCanvas.transform.position = defaultUITransform.position;
		dialogueCanvas.transform.position = defaultUITransform.position;

		Renderer r = moviePlane.GetComponent<MeshRenderer>();
		movie = (MovieTexture)r.material.mainTexture;
		moviePlane.GetComponent<MeshRenderer> ().material.color = Color.black;
		movie.loop = true;

		//Set state intro
		STATE = STATE_LEARNING;
		timer = Time.time;

	}

	// Update is called once per frame
	void Update ()
	{
		switch (STATE) {

		case STATE_LEARNING:

			movScript.pause();
			//Learning phase in order to let the user adapt to the controls
			if (Input.GetKeyDown ("space")) {
				STATE = STATE_START;
			}
			break;

		case STATE_START:
		
			//Start of the thing. Check if it's the first run through or second
			//If it's the run through the view of Alex, do some shit to make that possible.

			if (alexView) {

				Debug.Log ("Changing to Alexis View");

				//Stop the movie from playing all the bloody time
				movie.Stop ();
				moviePlane.GetComponent<MeshRenderer> ().material.color = Color.black;

				//Change mesh of Alex to headless, reset skin
				AlexMesh.GetComponent<SkinnedMeshRenderer> ().sharedMesh = AlexWithoutHeadMesh.GetComponent<SkinnedMeshRenderer> ().sharedMesh;
				Alex.GetComponentInChildren<Renderer> ().material.mainTexture = alexNormal;

				//Change mesh of Doctor to headed
				DoctorMesh.GetComponent<SkinnedMeshRenderer> ().sharedMesh = DoctorWithHeadMesh.GetComponent<SkinnedMeshRenderer> ().sharedMesh; 

				//Reset the doctors grabby hands
				GrabObject leftArmGrabScript = Doctor.GetComponentsInChildren<GrabObject> () [0];
				leftArmGrabScript.ResetGrabbing ();

				GrabObject rightArmGrabScript = Doctor.GetComponentsInChildren<GrabObject> () [1];
				rightArmGrabScript.ResetGrabbing ();

				//Change viewpoint to Alex's
				camera.transform.position = alexViewObject.transform.position;
				camera.transform.rotation = alexViewObject.transform.rotation;

				UiCamera.transform.parent = null;

				//Initiate playback of movement from script
				movScript.playback ();

				//Start the bollocks anew
				STATE = STATE_APPLY_GEL;
				timer = Time.time;
				break;
			}
				
			Debug.Log ("Starting from Doctor's view");
			movScript.record ();

			STATE = STATE_APPLY_GEL;
			timer = Time.time;
			break;


		//Waiting for the gel to cross the gel area.
		case STATE_APPLY_GEL:

			//Check for tardyness
			if (Time.time - timer > 10) {
				speechCanvas.transform.position = oppositeWallTransform.position;
				speechCanvas.transform.rotation = oppositeWallTransform.rotation;
				speechButtonText.text = "Alex: Tok ikke dette \n litt lang tid? ";
			};

			if (setup) {
				Debug.Log ("STATE_APPLY_GEL: " + alexView.ToString());

				if (alexView) {
					speechCanvas.transform.position = doctorsSpeechTransform.position;
					speechCanvas.transform.rotation = doctorsSpeechTransform.rotation;
				} else {
					speechCanvas.transform.position = oppositeWallTransform.position;
					speechCanvas.transform.rotation = oppositeWallTransform.rotation;
				}

				speechButtonText.text = "Doktor: Hei. Velkommen til ultralydundersøkelse.\nDet første vi skal gjøre er å legge på en \n gel som hjelper med å få et klart bilde,\n og så skal vi ta en titt på fosteret. ";

				setup = false;
			}

			//Check if the hand is within scanning area
			if (RightHandScript.isWithinScanningArea || LeftHandScript.isWithinScanningArea){

				//Check if the gel is in hand
				if (RightHandScript.hasGelInHand || LeftHandScript.hasGelInHand) {

					Debug.Log ("Gel has entered target area");
					Alex.GetComponentInChildren<Renderer> ().material.mainTexture = alexGelledTex;

					speechCanvas.transform.position = defaultUITransform.position;

					STATE = STATE_QUESTION_FROM_ALEX;
					setup = true;
					timer = Time.time;
					break;
				}

				//Check if the transducer is in hand
				if (RightHandScript.hasTransducerInHand || LeftHandScript.hasTransducerInHand) {
				
					Debug.Log ("Transducer has entered target area");
					speechCanvas.transform.position = oppositeWallTransform.position;
					speechCanvas.transform.rotation = oppositeWallTransform.rotation;
					speechButtonText.text = "Alex: Uhm, har du ikke glemt noe nå? ";
					timer = Time.time;
					break;
				}
			}
			break;

		case STATE_QUESTION_FROM_ALEX:

			//Check for tardyness
			if (Time.time - timer > 14) {
				speechCanvas.transform.position = oppositeWallTransform.position;
				speechCanvas.transform.rotation = oppositeWallTransform.rotation;
				speechButtonText.text = "Alex: Er det noe gale? ";
			};

			if (setup) {
				//Set up answers to question
				Debug.Log ("STATE_QUESTION_FROM_ALEX: " + alexView.ToString());
				buttonOne.text = "Doktor: Da var det på \ntide å se på barnet. ";
				buttonTwo.text = "Doktor: Svaralternativ nummer to,\n bare for morro skyld. ";
				dialogueCanvas.transform.position = oppositeWallTransform.position;
				dialogueCanvas.transform.rotation = oppositeWallTransform.rotation;
				setup = false;
				break;
			}

			if (Time.time - dialogueButtonScript.firstPressed > 2.0){
				if (dialogueButtonScript.currentSelectedButtonID == 0) {

					dialogueCanvas.transform.position = defaultUITransform.position;

					if (alexView) {
						speechCanvas.transform.position = doctorsSpeechTransform.position;
						speechCanvas.transform.rotation = doctorsSpeechTransform.rotation;
						speechButtonText.text = buttonOne.text;
					
					} else { 
						speechCanvas.transform.position = oppositeWallTransform.position;
						speechCanvas.transform.rotation = oppositeWallTransform.rotation;
						speechButtonText.text = "Alex: Ah, okay! ";
					}
						
					STATE = STATE_SCAN;
					timer = Time.time;
					setup = true;
					break;
				} else if (dialogueButtonScript.currentSelectedButtonID == 1) {

					dialogueCanvas.transform.position = defaultUITransform.position;

					if (alexView) {
						speechCanvas.transform.position = doctorsSpeechTransform.position;
						speechCanvas.transform.rotation = doctorsSpeechTransform.rotation;
						speechButtonText.text = buttonTwo.text;

					} else { 
						speechCanvas.transform.position = oppositeWallTransform.position;
						speechCanvas.transform.rotation = oppositeWallTransform.rotation;
						speechButtonText.text = "Alex: Huh? Hva mener du? ";
					}
					STATE = STATE_SCAN;
					timer = Time.time;
					setup = true;
					break;
				}
			}
			break;

		//Waiting for the scanner to get within the scanning area
		case STATE_SCAN:

			alexIKScript.lookObj = UiCamera.GetComponent<Transform>();

			//Check if the hand is within scanning area
			if (RightHandScript.isWithinScanningArea || LeftHandScript.isWithinScanningArea){

				//Check if the transducer is in hand
				if (RightHandScript.hasTransducerInHand || LeftHandScript.hasTransducerInHand) {

					Debug.Log ("Transducer has correctly entered target area");
					STATE = STATE_SCANNING;
					timer = Time.time;
					break;
				}
			}
			break;
		
		case STATE_SCANNING:

			alexIKScript.lookObj = moviePlane.GetComponent<Transform>();

			if ((Time.time - timer) > 7) {
				STATE = STATE_END;
				timer = Time.time;
				movie.Stop();
				moviePlane.GetComponent<MeshRenderer> ().material.color = Color.black;
				break;
			}
				
			if (!movie.isPlaying) {
				movie.Play ();
				moviePlane.GetComponent<MeshRenderer> ().material.color = Color.white;
			}

			if (RightHandScript.isWithinScanningArea || LeftHandScript.isWithinScanningArea) {
				break;
			}
			movie.Stop();
			moviePlane.GetComponent<MeshRenderer> ().material.color = Color.black;
			STATE = STATE_SCAN;
			break;

		case STATE_END:

			//Endcase.

			if (setup) {

				Debug.Log ("STATE_END: " + alexView.ToString());

				alexIKScript.lookObj = UiCamera.GetComponent<Transform>();

				if (alexView) {
					speechCanvas.transform.position = doctorsSpeechTransform.position;
					speechCanvas.transform.rotation = doctorsSpeechTransform.rotation;
				} else {
					speechCanvas.transform.position = oppositeWallTransform.position;
					speechCanvas.transform.rotation = oppositeWallTransform.rotation;
				}
				speechButtonText.text = "Doktor: Det var det. \nAlt virker som det er i orden med forsteret. \nDa skal jeg bare legge fra meg apparatet så er vi ferdig. ";
				setup = false;
			}

			if (RightHandScript.hasTransducerInHand || LeftHandScript.hasTransducerInHand || RightHandScript.hasGelInHand || LeftHandScript.hasGelInHand) {

				//Items still in hand, wait.
				break;
			}

			alexView = true;
			setup = true;
			STATE = STATE_LEARNING;
			speechCanvas.transform.position = defaultUITransform.position;
			break;
		}
	}
		
}


