﻿using UnityEngine;
using System.Collections.Generic;
using System;

public class StoreMovementScript : MonoBehaviour {

	private GameObject RightHandTarget;
	private GameObject LeftHandTarget;
	public Transform HeadTarget = null;

	private State scriptState;

	enum State{
		pause,
		recording,
		playback
	};

	struct transformStruct{
		public Vector3 position;
		public Quaternion rotation;
	}

	//Make a queue for the movements
	private Queue<transformStruct> RightHandTransformQueue = new Queue<transformStruct>();
	private Queue<transformStruct> LeftHandTransformQueue = new Queue<transformStruct>();
	private Queue<transformStruct> HeadTransformQueue = new Queue<transformStruct>();

	private transformStruct tempTransform;
		
	// Use this for initialization
	void Start () {
		scriptState = State.pause;
		RightHandTarget = GameObject.Find("TargetRightHand");
		LeftHandTarget = GameObject.Find("TargetLeftHand");
	}

	public void pause(){

		scriptState = State.pause;
		//Debug.Log("State: Pause");
	}

	public void record(){
		scriptState = State.recording;
		//Debug.Log("State: Recording");
	}

	public void playback(){
		scriptState = State.playback;
		//Debug.Log("State: Playback");

		//Access tracker scripts and disable tracking
		trackerInstance rTracker = RightHandTarget.GetComponent<trackerInstance>();
		rTracker.ApplyTracking = trackerInstance.Transform_Type.None;

		trackerInstance lTracker = LeftHandTarget.GetComponent<trackerInstance>();
		lTracker.ApplyTracking = trackerInstance.Transform_Type.None;
	}
	
	// Update is called once per frame
	void Update () {

		switch (scriptState){

		case State.pause:

			break;

		case State.recording:
			tempTransform.position = RightHandTarget.transform.position;
			tempTransform.rotation = RightHandTarget.transform.rotation;
			RightHandTransformQueue.Enqueue(tempTransform);

			tempTransform.position = LeftHandTarget.transform.position;
			tempTransform.rotation = LeftHandTarget.transform.rotation;
			LeftHandTransformQueue.Enqueue(tempTransform);

			tempTransform.position = HeadTarget.transform.position;
			tempTransform.rotation = HeadTarget.transform.rotation;
			HeadTransformQueue.Enqueue(tempTransform);
			break;

		case State.playback:
			if (RightHandTransformQueue.Count > 0){
				tempTransform = RightHandTransformQueue.Dequeue();
				RightHandTarget.transform.position = tempTransform.position;
				RightHandTarget.transform.rotation = tempTransform.rotation;

				tempTransform = LeftHandTransformQueue.Dequeue();
				LeftHandTarget.transform.position = tempTransform.position;
				LeftHandTarget.transform.rotation = tempTransform.rotation;

				tempTransform = HeadTransformQueue.Dequeue();
				HeadTarget.transform.position = tempTransform.position;
				HeadTarget.transform.rotation = tempTransform.rotation;
			}

			break;
		}
	}

	void OnDisable(){
		//Disable!
	}
}
