#define DllExport   __declspec( dllexport )

#include <math.h>
#include <string>
#include <map>
#include <iostream>

#include "trackerInstance.h"

using namespace std;

extern "C"{

	typedef std::map<std::string, VRPNTrackerInstance* > NameToTrackerMap;

	//Map over tracker instances
	static NameToTrackerMap trackers;

	//Start connection to the VRPN server
	void DllExport VRPNServerStart(const char* location)
	{
		printf("Connecting to running vrpn server %s\n", location);
	}

	//Run tracker and connection mainloops
	void DllExport VRPNServerLoop()
	{
		//iterate through the trackers and execute their mainloop
		NameToTrackerMap::iterator tkrIter = trackers.begin();
		while (tkrIter != trackers.end())
		{
			VRPNTrackerInstance* tkr = (*tkrIter).second;
			tkr->getTracker()->mainloop();
			tkrIter++;
		}
	}

	//Called when server stops in order to clean up garbage
	void DllExport VRPNServerStop()
	{
		//iterate through the trackers and delete them
		NameToTrackerMap::iterator tkrIter = trackers.begin();
		while (tkrIter != trackers.end())
		{
			VRPNTrackerInstance* tkr = (*tkrIter).second;
			delete tkr;
			tkrIter++;
		}
		trackers.erase(trackers.begin(), trackers.end());
	}

	//Called by the script attached to the object which is supposed to be transformed
	int DllExport VRPNTrackerStart(const char* name, int type = VRPN_POS, int max = 1000)
	{
		int debug = 0;
		std::string device(name);
		//find if remote object exists
		VRPNTrackerInstance* tkr = trackers[device];
		//if not, create a remote tracker object
		if (tkr == NULL)
		{
			tkr = new VRPNTrackerInstance(name);
			trackers[device] = tkr;
			debug = 1;
		}
		tkr->setMaxReports(max);
		tkr->setReportTypes(type);

		return debug;
	}

	//FINN UT HVOR DENNE KALLES
	void DllExport VRPNTrackerPosReport(const char* name, vrpn_TRACKERCB* cpy, timeval* ts, int sensor = 0)
	{
		std::string device(name);
		VRPNTrackerInstance* tkr = trackers[device];
		if (tkr)
		{
			tkr->getPosReport(cpy, ts, sensor);
		}
	}

	//FINN UT HVOR DENNE KALLES
	int DllExport VRPNTrackerNumPosReports(const char* name)
	{
		std::string device(name);
		VRPNTrackerInstance* tkr = trackers[device];
		if (tkr)
		{
			return tkr->getNumPosReports();
		}
		return 0;
	}

	//FINN UT HVOR DENNE KALLES
	void DllExport VRPNTrackerPosReports(const char* name, vrpn_TRACKERCB** cpy, int& cnt)
	{
		std::string device(name);
		VRPNTrackerInstance* tkr = trackers[device];
		if (tkr != NULL)
		{
			tkr->getPosReports(cpy, cnt);
		}
	}

	//FINN UT HVOR DENNE KALLES
	bool DllExport VRPNTimevalGreater(timeval* tv1, timeval* tv2)
	{
		return vrpn_TimevalGreater(*tv1, *tv2);
	}

}