#include "trackerInstance.h"

void VRPN_CALLBACK handle_pos(void *userdata, const vrpn_TRACKERCB t)
{
	VRPNTrackerInstance *tracker = (VRPNTrackerInstance*)userdata;
	tracker->addPosReport(t);
}

VRPNTrackerInstance::VRPNTrackerInstance(const char* name)
{
	tracker = new vrpn_Tracker_Remote(name);
	types = 0;
	maxReports = 1000;
}

VRPNTrackerInstance::~VRPNTrackerInstance()
{
	purgeReports();
	delete tracker;
}

void VRPNTrackerInstance::purgeReports()
{
	TrackerPosList::iterator PosItr;
	for (PosItr = pos.begin(); PosItr != pos.end(); PosItr++)
	{
		vrpn_TRACKERCB* curr = *PosItr;
		delete curr;
	}
	pos.erase(pos.begin(), pos.end());
}

vrpn_Tracker_Remote* VRPNTrackerInstance::getTracker()
{
	return tracker;
}

int VRPNTrackerInstance::getReportTypes()
{
	return types;
}

void VRPNTrackerInstance::setReportTypes(int in_types)
{
	if ((in_types & 1) && !(types & 1))
	{
		tracker->register_change_handler(this, handle_pos);
	}
	else if (!(in_types & 1) && (types & 1))
	{
		tracker->unregister_change_handler(this, handle_pos);
	}
}

void VRPNTrackerInstance::setMaxReports(int max)
{
	maxReports = max;
}

void VRPNTrackerInstance::addPosReport(const vrpn_TRACKERCB& rep)
{
	vrpn_TRACKERCB* cpy = new vrpn_TRACKERCB;
	*cpy = rep;
	if (cpy->msg_time.tv_sec == 0.0)
		vrpn_gettimeofday(&cpy->msg_time, NULL);
	pos.push_front(cpy);
	if ((int)pos.size() > maxReports)
	{
		vrpn_TRACKERCB* rem = pos.back();
		pos.pop_back();
		delete rem;
	}
}

void VRPNTrackerInstance::getPosReport(vrpn_TRACKERCB* cpy, timeval* ts, int in_sensor)
{
	TrackerPosList::iterator it;
	vrpn_TRACKERCB* last = NULL;
	for (it = pos.begin(); it != pos.end(); it++)
	{
		vrpn_TRACKERCB* curr = *it;
		if (curr->sensor == in_sensor)
		{
			if (ts == NULL)
			{
				*cpy = *curr;
				return;
			}
			else if (vrpn_TimevalGreater(*ts, curr->msg_time))
			{
				if (last)
				{
					double val = vrpn_TimevalMsecs(vrpn_TimevalDiff(*ts, curr->msg_time)) / vrpn_TimevalMsecs(vrpn_TimevalDiff(last->msg_time, curr->msg_time));
					cpy->pos[0] = curr->pos[0] + val*(last->pos[0] - curr->pos[0]);
					cpy->pos[1] = curr->pos[1] + val*(last->pos[1] - curr->pos[1]);
					cpy->pos[2] = curr->pos[2] + val*(last->pos[2] - curr->pos[2]);
					q_slerp(cpy->quat, curr->quat, last->quat, val);
					return;
				}
				else
				{
					*cpy = *curr;
					return;
				}
			}
			else
				last = curr;
		}
	}
}

int VRPNTrackerInstance::getNumPosReports()
{
	return (int)pos.size();
}


void VRPNTrackerInstance::getPosReports(vrpn_TRACKERCB** cpy, int& cnt)
{
	TrackerPosList::iterator it;
	int i = 0;
	for (it = pos.begin(); it != pos.end(); it++)
	{
		if (i < cnt)
		{
			vrpn_TRACKERCB* curr = *it;
			*cpy[i] = *curr;
			i++;
		}
		else
			break;
	}
	cnt = i;
}
